#!/usr/bin/env python

import os
from os import path
import unittest
from unittest import mock

from click.testing import CliRunner
from tldr import cli


ROOT = path.dirname(path.realpath(__file__))


class BasicTestCase(unittest.TestCase):
    def setUp(self):
        self.repo_dir = path.join(ROOT, 'mock_tldr')
        self.config_path = path.join(self.repo_dir, '.tldrrc')
        os.environ['TLDR_CONFIG_DIR'] = self.repo_dir
        self.runner = CliRunner()
        self.call_init_command()

    def tearDown(self):
        if path.exists(self.config_path):
            os.remove(self.config_path)

    def call_init_command(self, repo_dir=path.join(ROOT, 'mock_tldr'),
                          platform='linux'):
        with mock.patch('click.prompt', side_effect=[repo_dir, platform]):
            result = self.runner.invoke(cli.init)
        return result

    def call_update_command(self):
        with mock.patch('tldr.cli.build_index', return_value=None):
            with mock.patch('subprocess.call', return_value=0):
                result = self.runner.invoke(cli.update)
        return result

    def call_find_command(self, command_name, platform):
        command_args = (
            [command_name, '--on', platform] if platform else [command_name])
        result = self.runner.invoke(cli.find, command_args)
        return result

    def call_reindex_command(self):
        result = self.runner.invoke(cli.reindex)
        return result

    def call_locate_command(self, command_name, platform):
        command_args = (
            [command_name, '--on', platform] if platform else [command_name])
        result = self.runner.invoke(cli.locate, command_args)
        return result

    def call_list_command(self, platform):
        command_args = (['--on', platform] if platform else [])
        result = self.runner.invoke(cli.list, command_args)
        return result



#!/usr/bin/env python

import os
from os import path
import shutil

from basic import BasicTestCase


class TestReindex(BasicTestCase):
    def setUp(self):
        super().setUp()

        # Add a new page.
        self.page_path = path.join(self.repo_dir, 'pages')
        self.new_page = path.join(self.page_path, 'linux', 'blabla.md')
        shutil.copy(path.join(self.page_path, 'linux', 'tcpflow.md'),
                    self.new_page)

        # Backup the index.json.
        shutil.copy(path.join(self.page_path, 'index.json'),
                    path.join(self.page_path, 'index_bak.json'))

    def tearDown(self):
        super().tearDown()
        if path.exists(self.new_page):
            os.remove(self.new_page)
        # Restore the index.json.
        if path.exists(path.join(self.page_path, 'index_bak.json')):
            shutil.move(path.join(self.page_path, 'index_bak.json'),
                        path.join(self.page_path, 'index.json'))

    def test_reindex(self):
        before_reindex = self.call_find_command('blabla', platform='')
        assert 'Sorry' in before_reindex.output
        self.call_reindex_command()
        after_reindex = self.call_find_command('blabla', platform='')
        assert 'tcpflow' in after_reindex.output



#!/usr/bin/env python

import os
from os import path

import yaml

from basic import BasicTestCase


class TestInit(BasicTestCase):
    def test_init(self):
        assert path.exists(self.config_path)

        expected_config = {
            'colors': {
                'command': 'cyan',
                'description': 'blue',
                'usage': 'green'
            },
            'platform': 'linux',
            'repo_directory': self.repo_dir
        }
        with open(self.config_path, encoding='utf-8') as f:
            config = yaml.safe_load(f)
        assert expected_config == config

    def test_wrong_platform_input(self):
        result = self.call_init_command()
        assert result.output == (
            "There is already a config file exists, skip initializing it.\n"
        )

    def test_bad_repo_location(self):
        os.remove(self.config_path)
        result = self.call_init_command(repo_dir='/notexist')
        assert result.exception.args[0] == (
            "Repo path not exist, clone it first."
        )

    def test_back_platform_input(self):
        os.remove(self.config_path)
        result = self.call_init_command(platform='myos')
        assert result.exception.args[0] == (
            "Platform should be linux, osx or sunos."
        )



#!/usr/bin/env python

from unittest import mock

from tldr.config import get_config
from basic import BasicTestCase


class TestConfig(BasicTestCase):
    def test_config_not_exist(self):
        with mock.patch('os.path.exists', side_effect=[False, True]):
            self._assert_exception_message(
                ("Can't find config file at: {0}. You may use `tldr init` to "
                 "init the config file.").format(self.config_path)
            )

    def test_invalid_yaml_file(self):
        with mock.patch('builtins.open',
                        mock.mock_open(read_data="%YAML:1.0\nname:jhon")):
            self._assert_exception_message(
                "The config file is not a valid YAML file."
            )

    def test_unsupported_color_in_config(self):
        mock_config = {
            'colors': {
                'command': 'indigo',
                'description': 'cyan',
                'usage': 'green'
            },
            'platform': 'linux',
            'repo_directory': self.repo_dir
        }
        with mock.patch('yaml.safe_load', return_value=mock_config):
            self._assert_exception_message(
                "Unsupported colors in config file: indigo."
            )

    def test_repo_directory_not_exist(self):
        with mock.patch('os.path.exists', side_effect=[True, False]):
            self._assert_exception_message(
                "Can't find the tldr repo, check the `repo_directory` "
                "setting in config file."
            )

    def _assert_exception_message(self, expected_message):
        with self.assertRaises(SystemExit) as error:
            get_config()
        assert error.exception.args[0] == expected_message



#!/usr/bin/env python

from os import path

from basic import BasicTestCase


class TestLocate(BasicTestCase):
    def test_common_command(self):
        assert (self.call_locate_command('tldr', platform='').output.strip() ==
                path.join(self.repo_dir, 'pages', 'common', 'tldr.md'))



#!/usr/bin/env python

from basic import BasicTestCase


class TestFind(BasicTestCase):
    def test_find_tldr_in_common(self):
        self._assert_command_output(
            'tldr',
            ('\n  Simplified man pages\n\n- get typical usages of a command '
             '(hint: this is how you got here!)\n\n  tldr {{command}}\n\n')
        )

    def test_find_tcpflow_in_linux(self):
        self._assert_command_output(
            'tcpflow',
            ('\n  Capture TCP traffic for debugging and analysis\n\n'
             '- Show all data on the given interface and port\n\n'
             '  tcpflow -c -i {{eth0}} port {{80}}\n\n')
        )

    def test_find_airport_on_maxos(self):
        self._assert_command_output(
            'airport',
            ('\n  Wireless network configuration utility.\n\n'
             '- Show current wireless status information:\n\n'
             '  airport -I\n\n'
             '- Sniff wireless traffic on channel 1:\n\n'
             '  airport sniff {{1}}\n\n'
             '- Scan for available wireless networks:\n\n'
             '  airport -s\n\n'
             '- Disassociate from current airport network:\n\n'
             '  sudo airport -z\n\n'),
            'osx'
        )

    def test_can_not_find_something(self):
        self._assert_command_output(
            'yoooooooooooooo',
            ("Sorry, we don't support command: yoooooooooooooo right now.\n"
             "You can file an issue or send a PR on github:\n"
             "    https://github.com/tldr-pages/tldr\n")
        )

    def test_find_command_do_not_support_your_platform(self):
        self._assert_command_output(
            'airport',
            ('Sorry, command airport is not supported on your platform.\n'
             'You can file an issue or send a PR on github:\n'
             '    https://github.com/tldr-pages/tldr\n')
        )

    def _assert_command_output(self, command_name, expected_output,
                               platform=''):
        result = self.call_find_command(command_name, platform)
        assert result.output == expected_output



#!/usr/bin/env python

import unittest
from unittest import mock

from tldr.parser import parse_page


class TestParse(unittest.TestCase):
    def test_parse_page(self):
        mock_page = (
            '#node\n\n'
            '> Main node command\n\n'
            '- Call an interactive node shell\n\n'
            '`node`\n\n'
            '- Execute node on a JS file\n\n'
            '`node {{FILENAME}}.js`\n\n'
        )
        self._assert_mock_read(mock_page)

    def test_parse_new_syntax_page(self):
        mock_page = (
            'node\n'
            '====\n\n'
            '> Main node command\n\n'
            'Call an interactive node shell\n\n'
            '    node\n\n'
            'Execute node on a JS file\n\n'
            '    node {{FILENAME}}.js\n\n'
        )
        self._assert_mock_read(mock_page)

    def _assert_mock_read(self, page_content):
        mock_config = {
            'colors': {
                'command': 'cyan',
                'description': 'blue',
                'usage': 'green'
            },
            'platform': 'linux',
            'repo_directory': '/tmp/tldr'
        }
        expected_result = (
            '\n'
            '\x1b[0m\x1b[34m  Main node command\n'
            '\x1b[0m\n\x1b[0m\x1b[32m- Call an interactive node shell'
            '\n\x1b[0m\n\x1b[0m\x1b[36m  node\n'
            '\x1b[0m\n\x1b[0m\x1b[32m- Execute node on a JS file\n'
            '\x1b[0m\n\x1b[0m\x1b[36m  node {{FILENAME}}.js\n'
            '\x1b[0m\n\x1b[0m'
        )
        with mock.patch('tldr.parser.get_config', return_value=mock_config):
            with mock.patch('builtins.open', mock.mock_open(read_data=page_content)):
                result = parse_page('/repo_directory/pages/common/node.md')
                assert ''.join(result) == expected_result



#!/usr/bin/env python

from basic import BasicTestCase


class Testlist(BasicTestCase):
    def test_list_command(self):
        result = self.call_list_command('')
        assert result.output == 'du\ntcpflow\ntldr\n'

    def test_list_command_with_platform(self):
        result = self.call_list_command('osx')
        assert result.output == 'airport\ndu\ntldr\n'



#!/usr/bin/env python

import click

from tldr.config import get_config


def parse_page(page):
    """Parse the command man page."""
    colors = get_config()['colors']
    with open(page, encoding='utf-8') as f:
        lines = f.readlines()
    output_lines = []
    for line in lines[1:]:
        if is_headline(line):
            continue
        elif is_description(line):
            output_lines.append(click.style(line.replace('>', ' '),
                                            fg=colors['description']))
        elif is_old_usage(line):
            output_lines.append(click.style(line, fg=colors['usage']))
        elif is_code_example(line):
            line = '  ' + line if line.startswith('`') else line[2:]
            output_lines.append(click.style(line.replace('`', ''),
                                            fg=colors['command']))
        elif is_line_break(line):
            output_lines.append(click.style(line))
        else:
            output_lines.append(click.style('- ' + line, fg=colors['usage']))
    return output_lines


def is_headline(line):
    return line.startswith(('#', '='))


def is_description(line):
    return line.startswith('>')


def is_old_usage(line):
    return line.startswith('-')


def is_code_example(line):
    return line.startswith(('`', '    '))


def is_line_break(line):
    return line.startswith("\n")



#!/usr/bin/env python

from unittest import mock

from basic import BasicTestCase
from tldr import cli


class TestUpdate(BasicTestCase):
    def test_no_main_branch(self):
        with mock.patch('subprocess.call', return_value=1):
            result = self.runner.invoke(cli.update)
        assert result.exit_code != 0
        assert "renamed 'master' to 'main'" in result.output

    def test_update_available(self):
        mock_different_sha1 = [
            '8f82e7445fef6cc83c2e02b82df5f92fe0a909c6',
            'a4013ab1b14812624bbddf96feb1bfa2b03564f6'
        ]
        self._assert_update_info(mock_different_sha1, 'Updating...')

    def test_no_need_for_update(self):
        mock_same_sha1 = [
            '8f82e7445fef6cc83c2e02b82df5f92fe0a909c6',
            '8f82e7445fef6cc83c2e02b82df5f92fe0a909c6'
        ]
        self._assert_update_info(mock_same_sha1, 'No need for updates.')

    def _assert_update_info(self, mock_sha1, expected_message):
        with mock.patch('subprocess.check_output', side_effect=mock_sha1):
            with mock.patch('subprocess.check_call', side_effect=[0, 0]):
                result = self.call_update_command()
        assert expected_message in result.output
